/*____________________________________________________________________________
	
	Zinf - Zinf Is Not FreeA*p (The Free MP3 Player)

	Portions Copyright (C) 1999 EMusic.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	$Id: m3u.cpp,v 1.5 2003/05/11 06:38:18 kgk Exp $
____________________________________________________________________________*/

#include <assert.h>
#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>

#ifdef __QNX__
#include <strings.h>
#endif

using namespace std;

#include "config.h"
#include "errors.h"
#include "errno.h"
#include "utility.h"
#include "Http.h"

#include "m3u.h"

typedef struct FormatInfoStruct {
    const char* extension;
    const char* description;

} FormatInfoStruct; 

static FormatInfoStruct formats[] = {
    {"m3u", "M3U Playlist Format"},
    {"gqmpeg", "M3U Playlist Format"}
};

#define kNumFormats (sizeof(formats)/sizeof(FormatInfoStruct))

extern "C"
{
   PlaylistFormat *Initialize(FAContext* context)
   {
      return new M3U(context);
   }
}

M3U::M3U(FAContext* context):PlaylistFormat(context)
{
}

M3U::~M3U()
{

}

Error M3U::GetSupportedFormats(PlaylistFormatInfo* info, uint32_t index)
{
    Error result = kError_InvalidParam;

    assert(info);

    if(info)
    {
        result = kError_NoMoreFormats;

        if(index < kNumFormats)
        {
            info->SetExtension(formats[index].extension);
            info->SetDescription(formats[index].description);
            result = kError_NoErr;
        }
    }

    return result;
}

Error M3U::ReadPlaylist(const char* url, 
                        vector<PlaylistItem*>* list,
                        PLMCallBackFunction function,
                        void* cookie)
{
    Error result = kError_InvalidParam;

    assert(url);
    assert(list);

    if(url && list)
    {
	fstream fp;
        PlaylistItem* item;
        char* cp = NULL;
	
	string path;
	string entry;
	string root;
	int ind=-1;	
        bool cleanpath = false;

        result = URLToFilePath(url, path);

        if (IsError(result)) {
            // It's a url.. get the local 
            Http download(m_context);
            char buffer[TMP_MAX];
            cout << "download " << url << endl;
            tmpnam(buffer);
            path = buffer;
            result = download.DownloadToFile (url, path);
            cleanpath = true;
        }


        if(IsntError(result))
        {

	    root = path;

            cp = strrchr(root.c_str(), DIR_MARKER);
	    root.erase(root.length() - (strlen(cp) -1),(strlen(cp) +2));
            result = kError_FileNotFound;
	    fp.open(path.c_str(),ios::in);
	    
            if(fp){
                while(!fp.fail()) {
		    getline(fp,entry,'\n');
                    // is this a comment line?
                    if(entry.c_str()[0] == '#')
                        continue;

                    // if this is not a URL then let's
                    // enable people with different platforms 
                    // to swap files by changing the path 
                    // separator as necessary
                    if( strncmp(entry.c_str(), "http://", 7) &&
                        strncmp(entry.c_str(), "rtp://", 6) &&
                        strncmp(entry.c_str(), "file://", 7)) {

			while((ind = entry.find('\\'))>=0){
			    if(DIR_MARKER == '\\')
				break;
			    else
				entry.replace(ind,1,"/");
			}
			while((ind = entry.find('/'))>=0){
			    if(DIR_MARKER == '/')
				break;
			    else
				entry.replace(ind,1,"\\");
			}
                    }
		    for(ind = entry.length() -1;ind>=0;ind--){
			if(entry.c_str()[ind] == ' ')
			    entry.erase(ind,ind);
			else
			    break;
		    }    
		    
                    // is there anything left?
                    if(entry.length()) {
                        // is it a url already?
                        if (!strncmp(entry.c_str(), "http://", 7) ||
                            !strncmp(entry.c_str(), "rtp://", 6) ||
                            !strncmp(entry.c_str(), "file://", 7)) {
			    path = entry;
                        } else {
                            // is the path relative?
			    if(!entry.compare(0,2,"..") || 
			      (entry.compare(1,2,":\\") && 
			      entry.compare(0,1,DIR_MARKER_STR))){
			        path = root;
				path += entry;
                            } else
				path = entry;
                        
                            // make it a url so we can add it to the playlist
			    string itemurl;

                            if (IsntError(FilePathToURL(path.c_str(), itemurl)))
				path = itemurl;
                        }

                        item = new PlaylistItem(path.c_str());

                        list->push_back(item);
                    }
                }

                fp.close();

                result = kError_NoErr;
            }
        }
        if (cleanpath) unlink(path.c_str());
    }

    return result;
}

Error M3U::WritePlaylist(const char* url, PlaylistFormatInfo* format, 
                         vector<PlaylistItem*>* list,
                         PLMCallBackFunction function,
                         void* cookie)
{
    Error result = kError_InvalidParam;

    assert(url);
    assert(format);
    assert(list);

    if(url && format && list)
    {
        result = kError_FormatNotSupported;

        if(!strcasecmp("m3u", format->GetExtension()))
        {
            FILE* fp = NULL;
	    string path;
            URLToFilePath(url, path);

            result = kError_FileNoAccess;

            fp = fopen(path.c_str(), "wb");

            if(fp)
            {
                uint32_t index;
                uint32_t count;

                count = list->size();

                for(index = 0; index < count; index++)
                {
                    PlaylistItem* item = (*list)[index];

                    if(IsError(URLToFilePath(item->URL().c_str(),path)))
                        fprintf(fp, "%s%s", item->URL().c_str(), LINE_END_MARKER_STR);
                    else
                        fprintf(fp, "%s%s", path.c_str(), LINE_END_MARKER_STR);
                }

                fclose(fp);

                result = kError_NoErr;
            }
            else
            {
                int err = errno;

                switch(err)
                {
                    case EACCES:
                        result = kError_FileNoAccess;
                        break;

                    case EEXIST:
                        result = kError_FileNoAccess;
                        break;

                    case EINVAL:
                        result = kError_FileNoAccess;
                        break;

                    case ENOENT:
                        result = kError_FileNoAccess;
                        break;
                }
            }
        }
    }

    return result;
}
