/*____________________________________________________________________________
	
	Zinf - Zinf Is Not FreeA*p (The Free MP3 Player)

	Portions Copyright (C) 1999 EMusic.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	$Id: pls.cpp,v 1.6 2003/05/11 06:38:18 kgk Exp $
____________________________________________________________________________*/

#include <assert.h>
#include <iostream>
#include <string>
#include <stdlib.h>

#ifdef __QNX__
#include <strings.h>
#endif

using namespace std;

#include "config.h"
#include "errors.h"
#include "errno.h"
#include "utility.h"

#include "pls.h"
#include "debug.h"
#include "path_max.h"
#include "Http.h"

typedef struct FormatInfoStruct {
    const char* extension;
    const char* description;

} FormatInfoStruct; 

static FormatInfoStruct formats[] = {
    {"pls", "PLS Playlist Format"}
};

#define kNumFormats (sizeof(formats)/sizeof(FormatInfoStruct))

extern "C"
{
   PlaylistFormat *Initialize(FAContext* context)
   {
      return new PLS(context);
   }
}

PLS::PLS(FAContext* context):PlaylistFormat(context)
{
}

PLS::~PLS()
{

}

Error PLS::GetSupportedFormats(PlaylistFormatInfo* info, uint32_t index)
{
    Error result = kError_InvalidParam;

    assert(info);

    if(info)
    {
        result = kError_NoMoreFormats;

        if(index < kNumFormats)
        {
            info->SetExtension(formats[index].extension);
            info->SetDescription(formats[index].description);
            result = kError_NoErr;
        }
    }

    return result;
}

Error PLS::ReadPlaylist(const char* url, 
                        vector<PlaylistItem*>* list,
                        PLMCallBackFunction function,
                        void* cookie)
{
    FILE  *fp = NULL;

//    char   path[_MAX_PATH];

//    char   file[_MAX_PATH];
//    char   title[_MAX_PATH];
    char   key[_MAX_PATH];
    char   value[_MAX_PATH];
//    char   root[_MAX_PATH];

    string path,file,root,title; //file,title,key,value,root;
    
    char  *cp = NULL;
    int32_t  len;

    assert(url);
    assert(list);

    Error result = URLToFilePath(url, path);

    bool cleanpath = false;

    if (IsError(result)) {
        // It's a url.. get the local 
        Http download(m_context);
        char buffer[TMP_MAX];
        cout << "download " << url << endl;
        tmpnam(buffer);
        path = buffer;
        result = download.DownloadToFile (url, path);
        cleanpath = true;
    }
    if(IsError(result))
        return result;

    //strcpy(root, path);
    root = path;
    cp = strrchr(root.c_str(), DIR_MARKER);
    if(cp)
        *(cp + 1) = 0x00;
    fp = fopen(path.c_str(), "rb");
    if (!fp)
       return kError_FileNotFound;
    len = -1;
    while(!feof(fp)){
        if (fscanf(fp, " [%[^]]] ", key))
            continue;
        if (fscanf(fp, "%[^=]=%[^\n\r] ", key, value)){
            if (strncmp(key, "File", 4) == 0){
                if (file.length()){
                    AddItem(list, file, title, len, root);
                    title = "";
                    len = -1;
                }
		file = value;
                continue;
            }
            if (strncmp(key, "Title", 5) == 0){
		title = value;
                continue;
            }
            if (strncmp(key, "Length", 6) == 0){
                len = atoi(value);
                continue;
            }
            continue;
        }
    }
    if (file.length())
       AddItem(list, file, title, len, root);
    fclose(fp);
    if (cleanpath) unlink(path.c_str());

    return kError_NoErr;
}

Error PLS::WritePlaylist(const char* url, PlaylistFormatInfo* format, 
                         vector<PlaylistItem*>* list,
                         PLMCallBackFunction function,
                         void* cookie)
{
    Error result = kError_InvalidParam;

    assert(url);
    assert(format);
    assert(list);

    if(url && format && list)
    {
        result = kError_FormatNotSupported;

        if(!strcasecmp("pls", format->GetExtension()))
        {
            FILE* fp = NULL;
	    string path;

            URLToFilePath(url, path);

            result = kError_FileNoAccess;

            fp = fopen(path.c_str(), "wb");

            if(fp)
            {
                uint32_t index;
                uint32_t count;

                count = list->size();

                for(index = 0; index < count; index++)
                {
                    PlaylistItem* item = (*list)[index];


                    if(IsError(URLToFilePath(item->URL().c_str(), path)))
                        fprintf(fp, "%s%s", item->URL().c_str(), LINE_END_MARKER_STR);
                    else
                        fprintf(fp, "%s%s", path.c_str(), LINE_END_MARKER_STR);
                }

                fclose(fp);

                result = kError_NoErr;
            }
            else
            {
                int err = errno;

                switch(err)
                {
                    case EACCES:
                        result = kError_FileNoAccess;
                        break;

                    case EEXIST:
                        result = kError_FileNoAccess;
                        break;

                    case EINVAL:
                        result = kError_FileNoAccess;
                        break;

                    case ENOENT:
                        result = kError_FileNoAccess;
                        break;
                }
            }
        }
    }

    return result;
}


void PLS::AddItem(vector<PlaylistItem*>* list, string &entry, 
                  string &title, int32_t len, string &root)
{
    int32_t         index;
    PlaylistItem *item;
    string path;

    // if this is not a URL then let's
    // enable people with different platforms 
    // to swap files by changing the path 
    // separator as necessary
    if( strncmp(entry.c_str(), "http://", 7) &&
        strncmp(entry.c_str(), "rtp://", 6) &&
        strncmp(entry.c_str(), "file://", 7))
    {
	while((index = entry.find('\\'))>=0) {
	    if(DIR_MARKER == '\\')
		break;
	    else
		entry.replace(index,1,"/");
	}
	while((index = entry.find('/')) >= 0) {
	    if(DIR_MARKER == '/')
		break;
	    else
		entry.replace(index,1,"\\");
	}
    }

    // get rid of nasty trailing whitespace
    for (index = entry.length() - 1; index >=0; index--){
	if(entry.c_str()[index] == ' ')
	    entry.erase(index,index);
	else
	    break;
    }

    // is there anything left?
    if(entry.length())
    {
        // is it a url already?
        if (!strncmp(entry.c_str(), "http://", 7) ||
            !strncmp(entry.c_str(), "rtp://", 6) ||
            !strncmp(entry.c_str(), "file://", 7)) {
	    path = entry;
        } else {
            // is the path relative?
	    if(!entry.compare(0,2,"..") ||
	      (entry.compare(1,2,":\\") &&
	      entry.compare(0,1,DIR_MARKER_STR))){
	        path = root;
		path += entry;
            } else
		path = entry;
            
            // make it a url so we can add it to the playlist
	    string itemurl;

            if (IsntError(FilePathToURL(path.c_str(), itemurl)))
		path = itemurl;
        }

        if (title.length() || len > 0)
        {
            MetaData oData;
           
            if (title.length())
               oData.SetTitle(title.c_str());
            if (len > 0)
               oData.SetTime(len);

            item = new PlaylistItem(path.c_str(), &oData);
        }
        else
            item = new PlaylistItem(path.c_str());

        list->push_back(item);
    }
}
